@@ -1,7 +1,10 @@
-# This file is Copyright (c) 2017-2020 Florent Kermarrec <florent@enjoy-digital.fr>
-# This file is Copyright (c) 2017 Pierre-Olivier Vauboin <po@lambdaconcept.com>
-# This file is Copyright (c) 2018 bunnie <bunnie@kosagi.com>
-# License: BSD
+#
+# This file is part of LiteSDCard.
+#
+# Copyright (c) 2017-2020 Florent Kermarrec <florent@enjoy-digital.fr>
+# Copyright (c) 2017 Pierre-Olivier Vauboin <po@lambdaconcept.com>
+# Copyright (c) 2018 bunnie <bunnie@kosagi.com>
+# SPDX-License-Identifier: BSD-2-Clause
 
 from migen import *
 from migen.genlib.cdc import MultiReg
@@ -10,8 +13,8 @@ from litex.soc.interconnect.csr import *
 from litex.soc.interconnect import stream
 
 from litesdcard.common import *
-from litesdcard.crc import CRC, CRCChecker
-from litesdcard.crc import CRCDownstreamChecker, CRCUpstreamInserter
+from litesdcard.crc import CRC
+from litesdcard.crc import CRC16Checker, CRC16Inserter
 
 # SDCore -------------------------------------------------------------------------------------------
 
@@ -44,9 +47,9 @@ class SDCore(Module, AutoCSR):
         block_count  = self.block_count.storage
 
         # CRC Inserter/Checkers --------------------------------------------------------------------
-        self.submodules.crc7_inserter  = crc7_inserter  = CRC(9, 7, 40)
-        self.submodules.crc16_inserter = crc16_inserter = CRCUpstreamInserter()
-        self.submodules.crc16_checker  = crc16_checker  = CRCDownstreamChecker()
+        self.submodules.crc7_inserter  = crc7_inserter  = CRC(polynom=0x9, taps=7, dw=40)
+        self.submodules.crc16_inserter = crc16_inserter = CRC16Inserter()
+        self.submodules.crc16_checker  = crc16_checker  = CRC16Checker()
         self.comb += self.sink.connect(crc16_inserter.sink)
         self.comb += crc16_checker.source.connect(self.source)
 
@@ -75,13 +78,13 @@ class SDCore(Module, AutoCSR):
                 data_done,
                 data_error,
                 data_timeout,
-                ~crc16_checker.valid)),
-            crc7_inserter.val.eq(Cat(
+                0)), # FIXME data_response CRC.
+            crc7_inserter.din.eq(Cat(
                 cmd_argument,
                 cmd_command[8:14],
                 1,
                 0)),
-            crc7_inserter.clr.eq(1),
+            crc7_inserter.reset.eq(1),
             crc7_inserter.enable.eq(1),
         ]
 
@@ -130,7 +133,7 @@ class SDCore(Module, AutoCSR):
             phy.cmdr.sink.valid.eq(1),
             phy.cmdr.sink.last.eq(data_type == SDCARD_CTRL_DATA_TRANSFER_NONE),
             If(cmd_type == SDCARD_CTRL_RESPONSE_LONG,
-                phy.cmdr.sink.length.eq(17) # 136bits
+                phy.cmdr.sink.length.eq(17+1) # 136bits + 8bit shift to expose expected 128-bit window to software
             ).Else(
                 phy.cmdr.sink.length.eq(6)  # 48bits
             ),
@@ -191,4 +194,4 @@ class SDCore(Module, AutoCSR):
                     NextState("IDLE")
                 )
             )
-        )
\ No newline at end of file
+        )
